/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: f2_vector.cc 9095 2009-02-22 22:00:31Z gawrilow $"

#include <polymake.h>
#include <Matrix.h>
#include <Integer.h>
#include <HasseDiagram.h>

namespace polymake { namespace graph {

Matrix<Integer> f2_vector(perl::Object p)
{
   const HasseDiagram HD(p);
   const int d=HD.dim();

   Matrix<Integer> F2(d,d);
   F2(0,0) = HD.node_range_of_dim(0).size();

   Graph<Directed> G(HD.graph());
   for (int i=1; i<d; ++i) {
      if (i>1) {
	 const sequence row_range=HD.node_range_of_dim(0,i-2);
	 for (Rows< AdjacencyMatrix< Graph<Directed> > >::iterator
		 r=rows(adjacency_matrix(G)).begin()+row_range.front(),
		 r_last=rows(adjacency_matrix(G)).begin()+row_range.back();  r<=r_last;  ++r) {
	    Graph<Directed>::out_adjacent_node_list::iterator e=r->begin();
	    for (int n=r->size(); n>0; --n) {
	       *r += G.out_adjacent_nodes(*e);
	       r->erase(e++);
	    }
	 }
      }
      for (int j=0; j<i; ++j) {
	 Integer cnt=0;
	 for (Entire<sequence>::iterator f=entire(HD.node_range_of_dim(j)); !f.at_end(); ++f)
	    cnt += G.out_degree(*f);
	 F2(i,j)=F2(j,i)=cnt;
      }
      F2(i,i) = HD.node_range_of_dim(i).size();
   }

   return F2;
}

Function4perl(&f2_vector, "f2_vector(FaceLattice)");

} }
