/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: birkhoff.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <permutations.h>
#include <polymake.h>
#include <Rational.h>
#include <Matrix.h>
#include <Array.h>

namespace polymake { namespace polytope {

   perl::Object birkhoff(int n, bool even)
   {
      perl::Object p("Polytope<Rational>");
      if (even)
	 p.set_description() << "Even Birkhoff polytope for n=" << n << endl;
      else
	 p.set_description() << "Birkhoff polytope for n=" << n << endl;

      Matrix<int> V(even ? Integer::fac(n)/2 : Integer::fac(n),  n*n+1);
      Rows< Matrix<int> >::iterator v_i=rows(V).begin();
  
      AllPermutations<> perms(n);
      for (AllPermutations<>::const_iterator perm=entire(perms);  !perm.at_end();  ++perm, ++v_i) {
	 *v_i = 1 | concat_rows( permutation_matrix<int>(*perm) );

	 // only every second permutation is even
	 if (even)
	    ++perm;
      }
      p.take("VERTICES") << V;
      return p;
   }

   UserFunction4perl("# Category: Producing from scratch\n"
		     "# Construct a VERY INTERESTING polytope.\n",
		     &birkhoff, "birkhoff($;$=0)");
} }
