/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: center.cc 9054 2009-02-08 10:55:41Z joswig $"

#include <polymake.h>
#include <Vector.h>
#include <SparseMatrix.h>
#include <transform.h>

namespace polymake { namespace polytope {

   template <typename Scalar>
   perl::Object center(perl::Object p_in)
   {
      // read some point in the relative interior
      const Vector<Scalar> point=p_in.give("REL_INT_POINT");
      const int d = point.dim();

      // check if point is affine
      if (!point[0])
	 throw std::runtime_error("relative interior point not affine");

      perl::Object p_out(perl::ObjectType::construct<Scalar>("Polytope"));
      p_out.set_description() << "Centered polytope transformed from " << p_in.name() << endl;

      SparseMatrix<Scalar> tau=unit_matrix<Scalar>(d);
      tau[0].slice(1)=-point.slice(1);
      transform(p_out, p_in, tau);

      p_out.take("CENTERED") << true;
      return p_out;
   }

   UserFunctionTemplate4perl("# Category: Transforming a polyhedron\n"
			     "# Make a polyhedron centered.\n"
			     "# Apply a linear transformation to a polyhedron such that a relatively interior point\n"
			     "#  (preferably the vertex barycenter) is moved to the origin $(1,0,...,0)$.\n",
			     "center<Scalar> (Polytope<Scalar>)");
} }
