/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: centroid.cc 8202 2008-07-07 10:40:11Z gawrilow $"

#include <Rational.h>
#include <polymake.h>
#include <Array.h>
#include <Matrix.h>
#include <linalg.h>

namespace polymake { namespace polytope {

   template <typename Coord, typename Triangulation>
   void centroid(perl::Object p, const Matrix<Coord>& Points, const Triangulation& tr)
   {
      Coord volume(0);
      Vector<Coord> centroid(Points.cols());
      int d=tr.front().size()-1;

      for (typename Entire<Triangulation>::const_iterator s=entire(tr); !s.at_end(); ++s) {
	 const Matrix<Coord> sim=Points.minor(*s,All);
	 Coord v=abs(det(sim));
	 volume +=v;
	 Vector<Coord> b(Points.cols());
	 for (typename Entire< Rows <Matrix<Coord> > >::const_iterator i=entire(rows(sim));!i.at_end(); ++i)
	    b+=*i;
	 centroid+=v*b;
      }
      centroid/=volume*(d+1);
      volume /= Integer::fac(d);

      p.take("CENTROID")<<centroid;
      p.take("VOLUME") << volume;
   }

   FunctionTemplate4perl("centroid(Polytope Matrix *) : void");
} }

