/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: coherency_index.cc 8100 2008-05-13 11:08:56Z sherrmann $"

#include <Rational.h>
#include <polymake.h>
#include <Matrix.h>

#include <Set.h>

namespace polymake {
namespace polytope {

Rational coherency_index_2(perl::Object t_w, perl::Object t_wprime, const Matrix<Rational> &points, const Vector<Rational> &w, const Vector<Rational> wprime)
{

  const Matrix<Rational> V_wall = t_w.give("VERTICES");
  const Set<int> w_ff = t_w.give("FAR_FACE");
  const Matrix<Rational> V_wprimeall = t_wprime.give("VERTICES");
  const Set<int> wprime_ff = t_wprime.give("FAR_FACE");


  const Matrix<Rational> V_w = V_wall.minor(~w_ff,~scalar2set(0));//FIXME: can be changed if BOUNDED_VERTICES can be got directly t_w.give("BOUNDED_VERTICES");
  const Matrix<Rational> V_wprime = V_wprimeall.minor(~wprime_ff,~scalar2set(0));// t_wprime.give("BOUNDED_VERTICES");
  const int n=points.rows();

  Rational index = 100000;
  for (Entire< Rows <Matrix <Rational> > >::const_iterator i = entire(rows(V_w)); !i.at_end(); ++i) {
    Rational max = 0;
    for (Entire< Rows <Matrix <Rational> > >::const_iterator j = entire(rows(V_wprime)); !j.at_end(); ++j) {
      Rational min = 100000;
      for (int k=0; k<n;++k) {
	const Rational r = points[k]*(*j) + wprime[k];
	if (r!=0) {
	  const Rational r2= (points[k]*(*i) +w[k])/r;
#ifdef POLY_DEBUG
	  cerr<<" "<<r2;
#endif
	  min = r2<min ? r2 : min;
	}
      }
#ifdef POLY_DEBUG
      cerr<<"min"<<min<<endl;
#endif
      max = min > max ? min : max;
    }
#ifdef POLY_DEBUG
    cerr<<"max"<<max<<endl;
#endif
    index = max < index ? max : index;
  }
  return index;
}

Rational coherency_index(perl::Object t_w, perl::Object t_wprime)
{
  const Matrix<Rational> ineqs = t_w.give("INEQUALITIES");
  const Matrix<Rational> ineqsprime = t_wprime.give("INEQUALITIES");

  const Matrix<Rational> verts = ineqs.minor(All,~scalar2set(0));
  const Matrix<Rational> vertsprime = ineqsprime.minor(All,~scalar2set(0));

  if (verts!=vertsprime)
      throw std::runtime_error("Tight spans are not of the same polytopes.");
  return coherency_index_2(t_w, t_wprime, verts, ineqs.col(0), ineqsprime.col(0));
}


UserFunction4perl("# Computes the coherency index of w w.r.t. wprime\n",
  	  &coherency_index_2,"coherency_index(Polytope Polytope $ $ $)");

UserFunction4perl("# Computes the coherency index of w w.r.t. wprime\n",
  	  &coherency_index,"coherency_index(Polytope Polytope)");

} }
