/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident"$Project: polymake $$Id: common_refinement.cc 8081 2008-04-30 12:13:30Z sherrmann $"

#include <list>
#include <polymake.h>
#include <Set.h>
#include <Matrix.h>
#include <Rational.h>
#include <Array.h>


namespace polymake { namespace polytope {
 
Array<Set <int> > common_refinement(const Matrix<Rational> &vertices, const Array<Set <int> > &sub1, const Array<Set<int> > &sub2, const int dim)
{

  int n_cells = 0;

  std::list< Set<int> > l;
  for(Entire< Array<Set <int> > >::const_iterator i=entire(sub1);!i.at_end();++i)
    for(Entire< Array<Set <int> > >::const_iterator j=entire(sub2);!j.at_end();++j) {
      const Set<int> intersection = (*i)*(*j);
      if (intersection.size()>dim) {
	perl::Object p("Polytope<Rational>");
	p.take("VERTICES") << vertices.minor(intersection,All);
	const int int_dim = p.give("DIM");
	if (int_dim == dim) {
	  l.push_front(intersection);
	  ++n_cells;
	}
      }
    }
  
  Array< Set<int> > refinement(n_cells);
  for (Entire< Array<Set <int> > >::iterator i=entire(refinement); !i.at_end(); ++i) {
    *i= l.front();
    l.pop_front();
  }

  return refinement;
}

perl::Object common_refinement1(perl::Object p1, perl::Object p2)
{
  int dim = p1.give("DIM");
  const Matrix<Rational> vert=p1.give("VERTICES");
  const Array< Set<int> > sub1=p1.give("POLYTOPAL_SUBDIVISION");
  const Array< Set<int> > sub2=p2.give("POLYTOPAL_SUBDIVISION");
  
  perl::Object p_out("Polytope<Rational>");//FIXME: p_out should become a copy of p1 if there exists a copy method
  //  p_out.remove("WEIGHTS"); FIXME (If there is a remove method).
  if (p1.exists("WEIGHTS") && p2.exists("WEIGHTS")) {
    const Vector<Rational> w1=p1.give("WEIGHTS");
    const Vector<Rational> w2=p2.give("WEIGHTS");
    p_out.take("WEIGHTS")<<(w1+w2);
  }
  p_out.take("VERTICES")<<vert;//FIXME
  p_out.take("POLYTOPAL_SUBDIVISION")<<common_refinement(vert,sub1,sub2,dim);
  return p_out;
}


   UserFunction4perl("# Computes the common refinement of two subdivisions of @points\n"
		     "# It is assumend that there exists a common refinement of the two subdivisions.\n"
		     "# @a dim is the dimension of the point configuration"
		     "# args: points subdivision_1 subdivision_2 dim\n"
		     "# @author Sven Herrmann\n",
		     &common_refinement,"common_refinement($ $ $ $)");

  UserFunction4perl("# Computes the common refinement of two subdivisions of the same polytope p1, p2.\n"
		     "# It is assumend that there exists a common refinement of the two subdivisions.\n"
		     "# It is not checked if p1 and p2 are indeed the same!"
		     "# args: p1 p1\n"
		     "# @author Sven Herrmann\n",
		     &common_refinement1,"common_refinement(Polytope Polytope)");

} }
