/* Copyright (c) 1997-2006
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/


#include <Rational.h>
#include <polymake.h>
#include <Matrix.h>
#include <transform.h>


namespace polymake { namespace polytope {


template <typename Scalar>
perl::Object facet_to_infinity(perl::Object p_in, const int inf_facet)
{

  perl::Object p_out(perl::ObjectType::construct<Scalar>("Polytope"));

  const Matrix<Scalar> facets=p_in.give("FACETS");

  const Vector<Scalar> inf_f= facets.row(inf_facet);
  const Matrix<Scalar> tau=null_space(inf_f);
  transform_section(p_out, p_in, "FACETS | INEQUALITIES", T(inf_f/tau));
  transform_section(p_out, p_in, "AFFINE_HULL | EQUATIONS", T(inf_f/tau));
  transform_section(p_out, p_in, "VERTICES", inv(inf_f/tau));
  
  p_out.take("BOUNDED") << false;

  return p_out;
}

  UserFunctionTemplate4perl("# Category: Producing a new polyhedron from others\n"
			    "# Make an affine transformation such that the $i$th facet is transformed to infinity\n"
			    "# args: i\n"
			    "# @author Sven Herrmann\n",
			    "facet_to_infinity<Scalar>(Polytope<Scalar> $)");

} }
