/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: incidence.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <polymake.h>
#include <Matrix.h>
#include <IncidenceMatrix.h>
#include <linalg.h>

namespace polymake { namespace polytope {

template <typename E, typename Matrix1, typename Matrix2> inline
IncidenceMatrix<>
incidence_matrix(const GenericMatrix<Matrix1,E>& R, const GenericMatrix<Matrix2,E>& C)
{
   return IncidenceMatrix<> (R.rows(), C.rows(),
			     attach_operation(product(rows(R), rows(C), operations::mul()),
					      operations::composed11<conv<E,bool>, operations::logical_not>()).begin());
}

template <typename Matrix1, typename Matrix2> inline
IncidenceMatrix<>
incidence_matrix(const GenericMatrix<Matrix1,double>& R, const GenericMatrix<Matrix2,double>& C)
{
   return incidence_matrix<double, Matrix1, Matrix2>(normalized(R), normalized(C));
}

FunctionTemplate4perl("incidence_matrix(Matrix,Matrix)");

} }
