/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: isomorphic_polytopes.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <polymake.h>
#include <graph_compare.h>
#include <IncidenceMatrix.h>

namespace polymake { namespace polytope {

   bool isomorphic(perl::Object p1, perl::Object p2)
   {
      const IncidenceMatrix<> M1=p1.give("VERTICES_IN_FACETS"), M2=p2.give("VERTICES_IN_FACETS");
      return graph::isomorphic(M1, M2);
   }

   bool is_self_dual(perl::Object p)
   {
      const IncidenceMatrix<> M=p.give("VERTICES_IN_FACETS");
      return graph::isomorphic(M, T(M));
   }

   std::pair< Array<int>, Array<int> >
   find_facet_vertex_permutations(perl::Object p1, perl::Object p2)
   {
      const IncidenceMatrix<> M1=p1.give("VERTICES_IN_FACETS"), M2=p2.give("VERTICES_IN_FACETS");
      return graph::find_row_col_permutation(M1,M2);
   }

   UserFunction4perl("# category: Comparing\n"
		     "# return: boolean\n"
		     "# Check whether the face lattices of two polytopes are isomorphic.\n"
		     "# The problem is reduced to graph isomorphism of the vertex-facet incidence graphs.\n",
		     &isomorphic, "isomorphic(Polytope,Polytope)");	// FIXME: CombinatorialPolytope in all signatures

   UserFunction4perl("# category: Comparing\n"
		     "# args: p1, p2\n"
		     "# return: [ Array<Int>, Array<int> ]\n"
		     "# Find the permutations of facets and vertices which maps the @a p1 to @a p2.\n"
		     "# The facet permutation is the first component of the return value.\n"
		     "#\n"
		     "# Only the combinatorial isomorphism is considered.\n"
		     "# If the polytopes are not isomorphic, an exception is thrown.\n",
		     &find_facet_vertex_permutations, "find_facet_vertex_permutations(Polytope,Polytope)");

   Function4perl(&is_self_dual, "is_self_dual(Polytope)");
} }
