/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: pile.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <Rational.h>
#include <Vector.h>
#include <Matrix.h>
#include <MultiDimCounter.h>
#include <polymake.h>

namespace polymake { namespace polytope {

perl::Object pile(const Vector<int>& sizes)
{
   perl::Object p("Polytope<Rational>");
   p.set_description() << "Lifted pile of cubes" << endl;

   const int d=sizes.size();
   int n=1;
   for (Entire< Vector<int> >::const_iterator s=entire(sizes); !s.at_end(); ++s)
      n *= *s+1;

   Matrix<Rational> V(n,d+2);
   const int factor(d*d);
   Vector<Rational> limits(sizes);
   limits/=2;
   Rows< Matrix<Rational> >::iterator V_i=rows(V).begin();

   for (MultiDimCounter<false,Rational> x(-limits, limits+ones_vector<Rational>(d)); !x.at_end(); ++x, ++V_i)
      *V_i = 1 | *x | (sqr(*x) / factor);

   p.take("VERTICES") << V;
   p.take("N_VERTICES") << n;
   return p;
}

UserFunction4perl("# Category: Producing from scratch\n"
		  "# Produce a (d+1)-dimensional polytope from a pile of cubes.\n"
		  "# Start with a d-dimensional pile of cubes.  Take a generic convex function\n"
		  "# to lift this polytopal complex to the boundary of a (d+1)-polytope.\n"
		  "# s_i gives the number of boxes in the i-th dimension.\n"
		  "# args: s_1, ... , s_d",
		  &pile,"pile(@)");

} }
