/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: rand_box.cc 8423 2008-09-30 08:59:37Z sherrmann $"

#include <polymake.h>
#include <Matrix.h>
#include <random_generators.h>

namespace polymake { namespace polytope {

perl::Object rand_box(int d, int n, int b, perl::OptionSet options){

  if (d<1 || n<1 || b<1) throw std::runtime_error("rand_box: 1 <= dim, #POINTS, b");


   UniformRNG<long> random(options["seed"] ? std::atol(options["seed"]) : random_seed());


   perl::Object p("Polytope<Rational>");

   p.take("AMBIENT_DIM") << d;
   Matrix<int> V(n,d+1);
   for (int i=0; i<n; ++i) {
      V(i,0) = 1;
      for (int k=1; k<=d; ++k) {
	 int r=random.get()%(b+1);
	 if (r<0) r+=b;
	 V(i,k)=r;
      }
   }
  
   p.take("POINTS") << V;
   return p;

}
  UserFunction4perl("# Category: Producing from scratch\n"
		    "# Convex hull of <n> points samled uniformly at random from  the integer points in the cube [0,<b>]^<dim>.\n"
		    "# args: dim, n, b [seed=>undef]\n",
		    &rand_box,"rand_box($$$ { seed => undef })");
} }
