/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: rand_sphere.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <Matrix.h>
#include <Vector.h>
#include <random_generators.h>
#include <polymake.h>

namespace polymake { namespace polytope {

perl::Object rand_sphere(int d, int n, perl::OptionSet options)
{
   if (d<2 || n<=d) {
      throw std::runtime_error("rand_sphere: 2 <= dim < #vertices\n");
   }
   long seed;
   if(!(options["seed"] >> seed)) seed=random_seed();

   perl::Object p("Polytope<Rational>");
   p.set_description() << "Random sphere of dimension " << d << "; seed=" << seed << endl;

   UniformRNG< Vector<double> > random_source(d, seed);

   Matrix<double> Points(n, d+1);
   Points.col(0).fill(1);
   std::copy(random_source.begin(), entire(rows(Points.minor(All, range(1,d)).top())));
   p.take("POINTS") << Points;
   p.take("AMBIENT_DIM") << d;
   p.take("BOUNDED") << true;
   return p;
}

UserFunction4perl("# Category: Producing from scratch\n"
		  "# Produce a @a d-dimensional polytope with @a n random vertices\n"
		  "# uniformly distributed on the unit sphere.\n"
		  "# args: d, n, [ seed => VALUE ]\n",
		  &rand_sphere, "rand_sphere($$ { seed => undef })");
} }
