/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: rand_vert.cc 8042 2008-03-28 10:10:23Z sherrmann $"


#include <polymake.h>
#include <Matrix.h>
#include <Set.h>
#include <RandomSubset.h>

namespace polymake { namespace polytope {


template <typename Coord>
Matrix<Coord> rand_vert(const Matrix<Coord> &V, int n_vertices_out, perl::OptionSet options)
{
   const int n_vertices_in=V.rows();

   long seed;
   if(!(options["seed"] >> seed)) seed=random_seed();

   if (n_vertices_out > n_vertices_in)
      throw std::runtime_error("number of output vertices too high\n");

   Set<int> random_vertices=select_random_subset(sequence(0,n_vertices_in), n_vertices_out, seed);
   return V.minor(random_vertices, All);
}
  UserFunctionTemplate4perl("# Category: clients Producing a new polyhedron from others\n"
			    "# Selects $n$ random vertices from @c verts.\n\n"
			    "# This can be used to produce random polytopes which are neither simple nor simplicial as follows.\n"
			    "# First produce a simple polytope (for instance at random, by using rand_sphere, rand, or unirand).\n"
			    "# Then use this client to choose among the vertices at random.  Generalizes random 0/1-polytopes.\n\n"
 "#  A @c seed value for the random number generator can be specified.\n"
			    "# args: verts n [ seed => VALUE ]\n",
			    "rand_vert(Matrix, $, { seed=>undef })");
}}
