/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: splits_in_subdivision.cc 8429 2008-10-09 19:07:43Z gawrilow $"

#include <Matrix.h>
#include <Rational.h>
#include <Set.h>
#include <polymake.h>
#include <Array.h>

namespace polymake { namespace polytope {

Set<int> splits_in_subdivision(perl::Object p, const Array< Set<int> >& subdivision, const Matrix<Rational>& splits)
{
  const Matrix<Rational> vert=p.give("VERTICES");
  const int n=vert.rows();  

  int n_splits=splits.rows();
  Set<int> split_set;

  for (int j=0; j<n_splits; ++j) {
    const Vector<Rational> a=splits.row(j);
    Set<int> left;
    Set<int> right;
    for (int k=0; k<n; ++k) {
      const Vector<Rational> v(vert.row(k));
      Rational val=a*v;
      if (val>=0) {
	left.insert(k);
      }
      if (val<=0) {
	right.insert(k);
      }
    }
    bool cut=false;
    for (Entire< Array <Set<int> > >::const_iterator k=entire(subdivision); !cut && !k.at_end();  ++k)
      if (incl(left,*k)==2 && incl(right,*k)==2) cut=true;
    if (!cut) split_set.insert(j);
  }
  return split_set;
}

  UserFunction4perl("# Category: Other\n"
		    "# Tests which of the @a splits are coarsenings of the @a subdivision\n"
		    "# args: subdivision splits\n"
		    "# @author Sven Herrmann\n",
		    &splits_in_subdivision,"splits_in_subdivision(Polytope $; Matrix=$_[0]->SPLITS)");
} }
