/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include <Rational.h>
#include <polymake.h>
#include <Matrix.h>


namespace polymake { namespace polytope {

perl::Object ts_thrackle_metric(const int n)
{
  Matrix<Rational> d(n,n);

  if (n<2) throw std::runtime_error("n >= 2 required");
  
  for (int i=1; i<=n; ++i)
    for (int j=i+1; j<=n; ++j)
      d(i-1,j-1) = d(j-1,i-1) = (j-i)*(n-(j-i));

  perl::Object p("TightSpan");
  
  p.take("METRIC") << d;

  return p;
}

  UserFunction4perl("# Category: Tight Span\n"
		    "# Metric maximizing the f-vector of the tight span for given number of points.\n"
		    "# Lifting function for the thrackle triangulation (see de Loera,\n"
		    "# Sturmfels and Thomas: Groebner Basis and triangultaions of the second hypersimplex)\n"
		    "# args: n\n",
		    &ts_thrackle_metric,"ts_thrackle_metric");

}}
