/* Copyright (c) 1997-2008
   Technische Universitaet Berlin, Germany
   Department of Mathematics,
   Research Group Algorithmic and Discrete Mathematics
   mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330 Boston, MA 02111-1307, USA.
*/

#ident "$Project: polymake $$Id: types.cc 8397 2008-08-28 13:48:21Z gawrilow $"

#include <polymake.h>
#include <Set.h>
#include <Array.h>
#include <Matrix.h>

namespace polymake { namespace tropical {

template <typename Coord>
Array< Array <Set <int> > >  types(const Matrix<Coord>& points, const Matrix<Coord>& generators)
{
   const int m(points.rows()), n(generators.rows()), d(generators.cols());
   Array< Array < Set<int> > > types(m);

   for (int j=0; j<m; ++j) { // determine the type of each point
      Array< Set<int> > this_type(d);

      for (int i=0; i<n; ++i) { // iterate over all generators
         Vector<Coord> diff = generators[i]-points[j];
         Coord m = accumulate(diff, operations::min());
         for (int k=0; k<d; ++k)
            if (diff[k]==m) this_type[k] += i;
      }

      types[j] = this_type;
   }

   return Array< Array< Set <int> > >(types);
}

UserFunctionTemplate4perl("# Compute the types of the @a generators set relative to a set of @a points.\n"
			  "#  The following are two typical cases:\n"
			  "#  (1) generators = VERTICES and points = VERTICES\n"
			  "#  (2) generators = POINTS  and points = PSEUDOVERTICES\n"
			  "# args: points generators\n",
			  "types<Coord>(Matrix<Coord> Matrix<Coord>)");
} }
