/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ifndef _POLYMAKE_LINALG_EXCEPTIONS_H
#define _POLYMAKE_LINALG_EXCEPTIONS_H "$Project: polymake $$Id: linalg_exceptions.h 8921 2008-11-17 12:29:14Z gawrilow $"

#include <stdexcept>

namespace pm {
   class linalg_error : public std::runtime_error {
   public:
      linalg_error(const std::string& what) : std::runtime_error(what) {}
   };

   class degenerate_matrix : public linalg_error {
   public:
      degenerate_matrix() : linalg_error("matrix not invertible") {}
      degenerate_matrix(const std::string& what) : linalg_error(what) {}
   };

   class infeasible : public linalg_error {
   public:
      infeasible() : linalg_error("infeasible linear equations system") {}
      infeasible(const std::string& what) : linalg_error(what) {}
   };

   class unbounded : public linalg_error {
   public:
      unbounded() : linalg_error("unbounded linear programming problem") {}
      unbounded(const std::string& what) : linalg_error(what) {}
   };

   class not_pointed : public linalg_error {
      int dim;
   public:
      not_pointed(int dim_arg=0) : linalg_error("polyhedron not pointed"), dim(dim_arg) {}
      not_pointed(const std::string& what) : linalg_error(what), dim(0) {}

      int get_dim() const { return dim; }
   };
} // end namespace pm

namespace polymake {
   using pm::linalg_error;
   using pm::degenerate_matrix;
   using pm::infeasible;
   using pm::unbounded;
   using pm::not_pointed;
}

#endif // _POLYMAKE_LINALG_EXCEPTIONS_H

// Local Variables:
// mode:C++
// End:
