/* Copyright (c) 1997-2006
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ifndef _POLYMAKE_NOTHING_H
#define _POLYMAKE_NOTHING_H "$Project: polymake $$Id: nothing.h 7683 2007-05-23 22:06:25Z gawrilow $"

#ifndef _POLYMAKE_TYPE_MANIP_H
# include <type_manip.h>
#endif

namespace pm {

/// Structure denoting the absence of data
struct nothing {
   void operator= (const nothing&) const { }
   bool operator== (const nothing&) const { return true; }
   bool operator!= (const nothing&) const { return false; }
   bool operator< (const nothing&) const { return false; }
   bool operator> (const nothing&) const { return false; }
   bool operator<= (const nothing&) const { return true; }
   bool operator>= (const nothing&) const { return true; }
};

inline void relocate(const nothing*, const nothing*) { }

template <>
struct spec_object_traits<nothing> : spec_object_traits<is_scalar> {
   typedef nothing model;	// prevent mixing this with any other type
};

/// Masquerade class for suppressing I/O and conversion
template <typename T, typename Expected=nothing>
struct ignore { };

} // end namespace pm

namespace polymake {
   using pm::nothing;
}

namespace std {
   /// Do nothing
   inline void swap(const pm::nothing&, const pm::nothing&) { }
}

#endif // _POLYMAKE_NOTHING_H

// Local Variables:
// mode:C++
// End:
