/* Copyright (c) 1997-2009			 -*- C -*-
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: Object.xs 9026 2009-01-09 18:03:47Z gawrilow $"

#include "Ext.h"

int pm_perl_Object_name_index, pm_perl_Object_description_index,
    pm_perl_Object_parent_index, pm_perl_Object_transaction_index, pm_perl_Object_attachments_index;

MODULE = Polymake::Core::Object			PACKAGE = Polymake::Core::Object

PROTOTYPES: DISABLE

void
_prop_set_accessor(prop)
   SV *prop;
CODE:
{
   PERL_CONTEXT *cx_bottom=cxstack, *cx_top=cx_bottom+cxstack_ix, *cx=cx_top;
   OP *o;
   AV *args=GvAV(PL_defgv);
   U32 save_flags=AvREIFY(args);
   AvREIFY_off(args);
   av_push(args, prop);			/* pushed into the current @_ */
   while (cx >= cx_bottom) {
      if (CxTYPE(cx)==CXt_SUB) {
	 cx->blk_sub.lval=0;
	 if (!SkipDebugFrame(cx,1)) {
	    o=ReturnsToOp(cx);
	    if (!o) break;	/* called from call_sv due to some magic: assume a read access */
	    if (o->op_type != OP_LEAVESUB && o->op_type != OP_LEAVESUBLV) {
	       if (o->op_type == OP_SASSIGN && !(o->op_private & OPpASSIGN_BACKWARDS)) {
		  SV *rhs=*(PL_stack_base+cx->blk_oldsp);		/* value to be assigned ... */
		  av_push(args, rhs);
		  if (save_flags) AvREIFY_on(args);
		  ReturnsToOp(cx)=o->op_next;	/* skip pp_sassign */
		  XSRETURN_YES;
	       }
	       break;
	    }
	 }
      }
      --cx;
   }
   if (save_flags) AvREIFY_on(args);
   XSRETURN_NO;
}


void
_get_alternatives(...)
PPCODE:
{
   SV *descend_path= items==1 ? ST(0) : Nullsv;
   PERL_CONTEXT *cx_bottom=cxstack, *cx=cx_bottom+cxstack_ix;
   while (cx >= cx_bottom) {
      if (CxTYPE(cx)==CXt_SUB && !SkipDebugFrame(cx,1)) {
	 OP *o=ReturnsToOp(cx);
	 if (!o) break;		/* called from call_sv due to some magic: assume no alternatives */
	 if (o->op_type != OP_LEAVESUB && o->op_type != OP_LEAVESUBLV) {
	    I32 skip=FALSE, push= GIMME_V == G_ARRAY;
#ifdef USE_ITHREADS
	    SV **saved_curpad=0;
#endif
	    OP *nop=o, *gvop;
	    AV *path_av=0;
	    if (descend_path) {
	       while (nop->op_type == OP_METHOD_NAMED && nop->op_next->op_type == OP_ENTERSUB) {
		  if (!path_av) {
		     path_av=newAV();
		     AvREAL_off(path_av);
		     sv_upgrade(descend_path, SVt_RV);
		     SvRV_set(descend_path, (SV*)path_av);
		     SvROK_on(descend_path);
		  }
#ifdef USE_ITHREADS
		  if (!saved_curpad) {
		     saved_curpad=PL_curpad;
		     PL_curpad=pm_perl_get_cx_curpad(aTHX_ cx, cx_bottom);
		  }
#endif
		  av_push(path_av, cSVOPx_sv(nop));
		  nop=nop->op_next->op_next;
	       }
	    }
	    while ((nop->op_type == OP_CONST && (gvop=0, nop=nop->op_next)->op_type == OP_BIT_OR) ||
		   (nop->op_type == OP_PUSHMARK && (nop=nop->op_next)->op_type == OP_GV &&
		    (gvop=nop, nop=nop->op_next)->op_type == OP_ENTERSUB && (nop=nop->op_next)->op_type == OP_BIT_OR)) {
	       if (push) {
#ifdef USE_ITHREADS
		  if (!saved_curpad) {
		     saved_curpad=PL_curpad;
		     PL_curpad=pm_perl_get_cx_curpad(aTHX_ cx, cx_bottom);
		  }
#endif
		  if (gvop) {
		     GV *cgv=cGVOPx_gv(gvop);
		     XPUSHs(sv_2mortal(newSVpvn(GvNAME(cgv),GvNAMELEN(cgv))));
		  } else {
		     XPUSHs(cSVOPo_sv);
		  }
	       }
	       skip=TRUE;
	       o=nop=nop->op_next;
	    }
	    if (skip && !path_av) ReturnsToOp(cx)=o;
#ifdef USE_ITHREADS
	    if (saved_curpad) PL_curpad=saved_curpad;
#endif
	    break;
	 }
      }
      --cx;
   }
}

void
_expect_array_access()
PPCODE:
{
   PERL_CONTEXT *cx_bottom=cxstack, *cx_top=cx_bottom+cxstack_ix, *cx=cx_top;
   OP *o;
   while (cx >= cx_bottom) {
      if (CxTYPE(cx)==CXt_SUB) {
	 if (!SkipDebugFrame(cx,0)) {
	    o=ReturnsToOp(cx);
	    for (;;) {
	       if (!o) {
		  if (cx->blk_gimme==G_ARRAY) XSRETURN_YES;
		  else XSRETURN_NO;
	       }
	       if (o->op_type == OP_LEAVE) o=o->op_next;
	       else break;
	    }
	    if (o->op_type != OP_LEAVESUB && o->op_type != OP_LEAVESUBLV) {
	       if (o->op_type == OP_RV2AV) XSRETURN_YES;
	       break;
	    }
	 }
      }
      --cx;
   }
   XSRETURN_NO;
}

BOOT:
{
   pm_perl_Object_name_index=CvDEPTH(get_cv("Polymake::Core::Object::name", FALSE));
   pm_perl_Object_description_index=CvDEPTH(get_cv("Polymake::Core::Object::description", FALSE));
   pm_perl_Object_parent_index=CvDEPTH(get_cv("Polymake::Core::Object::parent", FALSE));
   pm_perl_Object_transaction_index=CvDEPTH(get_cv("Polymake::Core::Object::transaction", FALSE));
   pm_perl_Object_attachments_index=CvDEPTH(get_cv("Polymake::Core::Object::attachments", FALSE));
   if (PL_DBgv) {
      CvNODEBUG_on(get_cv("Polymake::Core::Object::_prop_set_accessor", FALSE));
      CvNODEBUG_on(get_cv("Polymake::Core::Object::_get_alternatives", FALSE));
      CvNODEBUG_on(get_cv("Polymake::Core::Object::_expect_array_access", FALSE));
   }
}
