#  Copyright (c) 1997-2009
#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
#  http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version: http://www.gnu.org/licenses/gpl.txt.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#-----------------------------------------------------------------------------
#  $Project: polymake $$Id: BackgroundProcess.pm 9026 2009-01-09 18:03:47Z gawrilow $

use strict;
use namespaces;

package Polymake::Background::Watcher;
use POSIX qw( :sys_wait_h );

my %active;	# pid => something to destroy on termination
my $init;

sub gather_zombies {
   my $flag=shift || 0;
   while (keys(%active) && (my $pid=waitpid(-1,$flag)) > 0) {
      delete $active{$pid};
   }
}

sub add_process {
   if ($init) {
      gather_zombies(WNOHANG);
   } else {
      add AtEnd("BackgroundProcess", \&gather_zombies);
      $init=1;
   }
   $active{$_[0]}=$_[1];
}

package Polymake::BackgroundProcess;

sub new {
   shift;
   my ($to_destroy, $to_close);
   while (1) {
      if ($_[0] eq "to_destroy") {
         shift;
	 $to_destroy=shift;
      } elsif ($_[0] eq "to_close" ) {
	 shift;
	 $to_close=shift;
      } else {
	 last;
      }
   }
   dbg_print( "starting @_" ) if $DebugLevel;
   my $pid=fork;
   if (!$pid) {
      die "BackgroundProcess: fork failed: $!\n" if !defined($pid);
      $to_close and $to_close->();
      STDIN->close;
      POSIX::setsid();
      exec @_
      or do {
	 my ($progname)= @_==1 ? $_[0] =~ /^\s*(\S+)/ : @_;
	 print STDERR "could not start $progname: $!\n";
	 POSIX::_exit(1);	# avoid executing global destructors
      }
   }
   Background::Watcher::add_process($pid, $to_destroy);
   return $pid;
}

package Polymake::BackgroundPipe;
use IPC::Open3;

sub new {
   shift;
   my ($out, $in);
   eval {
      Background::Watcher::add_process(open3($out, $in, ">&STDERR", @_));
   };
   if ($@) {
      $@ =~ s/^.*?://;
      $@ =~ s/ at \(eval.*$/\n/;
      if ($$ != getpgrp) {	# we are in the fork!!!
	 print STDERR $@;
	 POSIX::_exit(1);	# avoid executing global destructors
      }
      die $@;
   }
   $out->autoflush;
   ($out, $in);
}

1
